local UnitCastingInfo = UnitCastingInfo;
local UnitChannelInfo = UnitChannelInfo;
local GetSpellInfo = GetSpellInfo;
local GetSpellCooldown = GetSpellCooldown;
local UnitAura = UnitAura;
local GetItemCooldown = GetItemCooldown;
local GetRuneCooldown = GetRuneCooldown;
local GetRuneType = GetRuneType;
local UnitPower = UnitPower;
local UnitPowerMax = UnitPowerMax;
local UnitHealth = UnitHealth;
local UnitHealthMax = UnitHealthMax;
local select = select;
local string_match = string.match;

function Gnosis:Timers_Spell( bar, timer, ti )
	-- cast
	local spell, _, _, icon, s, d = UnitCastingInfo( timer.unit );
	if( d and d > 0 ) then
		if( timer.spell == "all" or timer.spell == spell ) then
			ti.bChannel = false;
			ti.dur = (d - s);
			ti.fin = d;
			ti.cname = spell;
			ti.icon = icon;
			ti.unit = timer.unit;
			ti.ok = true;
		end
	else
		spell, _, _, icon, s, d = UnitChannelInfo( timer.unit );
		if( d and d > 0 ) then
			if( timer.spell == "all" or timer.spell == spell ) then
				ti.bChannel = true;
				ti.dur = (d - s);
				ti.fin = d;
				ti.cname = spell;
				ti.icon = icon;
				ti.unit = timer.unit;
				ti.ok = true;
			end
		end
	end
end

function Gnosis:Timers_SpellCD( bar, timer, ti )
	-- cooldown, player only
	ti.unit = "player";
	local s, d = GetSpellCooldown( timer.spell );
	if( d and d > 1.5 ) then	-- duration greater than global cd
		ti.bChannel = true;
		ti.dur = d * 1000;
		ti.fin = (s + d) * 1000;
		ti.cname = timer.spell;
		ti.stacks = nil;
		ti.ok = true;
		ti.icon = select( 3, GetSpellInfo( timer.spell ) );
	elseif( timer.bNot ) then
		ti.bChannel = true;
		ti.cname = timer.spell;
		ti.stacks = nil;
		ti.icon = select( 3, GetSpellInfo( timer.spell ) );
		ti.dur = 1;
		ti.fin = 1;
		ti.ok = true;
		ti.bSpecial = true;
	end
end

function Gnosis:Timers_Aura( bar, timer, ti )
	-- aura == buff or debuff (== hot or dot)
	local _, _, ic, sta, _, d, s = UnitAura( timer.unit, timer.spell, nil, timer.filter )
	ti.unit = timer.unit;
	if( s and d ) then
		if( s > 0 ) then
			-- dynamic aura
			ti.dur = d * 1000;
			ti.fin = s * 1000;
			ti.ok = true;
		elseif( timer.nodur and s == 0 and d == 0 ) then
			-- static aura
			ti.dur = 1;
			ti.fin = 1;
			ti.ok = true;
			ti.bSpecial = true;
		end
		ti.bChannel = true;
		ti.cname = timer.spell;
		ti.stacks = (sta and sta > 0) and sta or nil;
		ti.icon = ic;
	elseif( timer.bNot ) then
		ti.bChannel = true;
		ti.cname = timer.spell;
		ti.stacks = nil;
		ti.icon = select( 3, GetSpellInfo( timer.spell ) );
		ti.dur = 1;
		ti.fin = 1;
		ti.ok = true;
		ti.bSpecial = true;
	end
end

function Gnosis:Timers_ItemCD( bar, timer, ti )
	-- itemcd, player only
	ti.unit = "player";
	if( not timer.iid ) then
		local _, link, _, _, _, _, _, _, _, itex = GetItemInfo( timer.spell );
		if( link and itex ) then
			local iid = string_match( link, "|Hitem:(%d+):" );
			timer.iid = iid;
			timer.itex = itex;
		end
	end

	if( timer.iid ) then
		local s, d = GetItemCooldown( timer.iid );
		if( d and d > 1.5 ) then	-- duration greater than global cd
			ti.bChannel = true;
			ti.dur = d * 1000;
			ti.fin = (s + d) * 1000;
			ti.cname = timer.spell;
			ti.icon = timer.itex;
			ti.ok = true;
			ti.stacks = nil;
		elseif( timer.bNot ) then
			ti.bChannel = true;
			ti.dur = 1;
			ti.fin = 1;
			ti.cname = timer.spell;
			ti.icon = timer.itex;
			ti.stacks = nil;
			ti.ok = true;
			ti.bSpecial = true;
		end
	end
end

function Gnosis:Timers_RuneCD( bar, timer, ti )
	-- rune cooldown, player only
	ti.unit = "player";
	local s, d, rdy = GetRuneCooldown( timer.spell );
	if( s and s > 0 ) then
		ti.bChannel = true;
		ti.dur = d * 1000;
		ti.fin = (s + d) * 1000;
		local rune = GetRuneType( timer.spell );
		if( rune ) then
			ti.cname = Gnosis.tRuneName[rune];
			ti.icon = Gnosis.tRuneTexture[rune];
		else
			ti.cname = "";
			ti.icon = nil;
		end
		ti.stacks = nil;
		ti.ok = true;
	elseif( timer.bNot and rdy ) then
		ti.bChannel = true;
		ti.dur = 1;
		ti.fin = 1;
		local rune = GetRuneType( timer.spell );
		if( rune ) then
			ti.cname = Gnosis.tRuneName[rune];
			ti.icon = Gnosis.tRuneTexture[rune];
		else
			ti.cname = "";
			ti.icon = nil;
		end
		ti.stacks = nil;
		ti.ok = true;
		ti.bSpecial = true;
	end
end

function Gnosis:Timers_Power( bar, timer, ti )
	local s, d = UnitPower( timer.unit ), UnitPowerMax( timer.unit );
	if( d and d > 0 ) then
		local pts = select( 2, UnitPowerType( timer.unit ) );
		ti.cname = pts and _G[pts] or "";
		ti.bChannel = true;
		ti.bSpecial = true;
		ti.fin = s;
		ti.dur = d;
		ti.unit = timer.unit;
		ti.icon = nil;
		ti.ok = true;
	end
end

function Gnosis:Timers_Health( bar, timer, ti )
	local s, d = UnitHealth( timer.unit ), UnitHealthMax( timer.unit );
	if( d and d > 0 ) then
		ti.cname = _G["HEALTH"];
		ti.bChannel = true;
		ti.bSpecial = true;
		ti.fin = s;
		ti.dur = d;
		ti.unit = timer.unit;
		ti.icon = nil;
		ti.ok = true;
	end
end

function Gnosis:Timers_PowerAlternate( bar, timer, ti )
	local s, d = UnitPower( timer.unit, ALTERNATE_POWER_INDEX ), UnitPowerMax( timer.unit, ALTERNATE_POWER_INDEX );
	if( d and d > 0 ) then
		ti.cname = select( 10, UnitAlternatePowerInfo( timer.unit ) )
		ti.bChannel = true;
		ti.bSpecial = true;
		ti.fin = s;
		ti.dur = d;
		ti.unit = timer.unit;
		ti.icon = UnitAlternatePowerTextureInfo( timer.unit, 2 );
		ti.ok = true;
	end
end
